/*
** This file is completely free, in every sense of the word.
** Do whatever you like with it, just beware that you use this
** code entirely at your own risk. I don't accept any liability
** whatsoever for any loss or damage caused by the use or
** misuse of this code.
**
** Jo Even Skarstein
** joska@nvg.org
** http://joska.nvg.org/
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <aes.h>
#include <mintbind.h>
#include "av.h"
#include "nugatti.h"

int gl_avserver_apid;
extern int gl_apid;

char *av_buffer = NULL;

/*
** VA_START
*/
int va_start(int apid, char *program, char *cmd)
{
	int msg[8] = {VA_START,0,0,0,0,0,0,0};

	if (!apid)
	{
		apid = gl_avserver_apid;
	}

	msg[1] = gl_apid;
	if (cmd != NULL)
	{
		sprintf(av_buffer, "%s %s", program, cmd);
	}
	else
	{
		sprintf(av_buffer, "%s", program);
	}
	AVSTR2MSG(msg, 3, av_buffer);

	return appl_write(apid, 16, msg);
}

/*
** AV_STARTPROG
*/
int av_startprog(char *program, char *cmd)
{
	int msg[8] = {AV_STARTPROG,0,0,0,0,0,0,0}, res;
	char *glProg = NULL, *glArg = NULL;

	glProg = av_buffer;
	glArg = glProg + 256;

	if (glProg && glArg)
	{
		msg[1] = gl_apid;

		if (program != NULL)
		{
			strncpy(glProg, program, 255);
			AVSTR2MSG(msg, 3, glProg);
		}

		if (cmd != NULL)
		{
			strncpy(glArg, cmd, 255);
			AVSTR2MSG(msg, 5, glArg);
		}
		res = appl_write(gl_avserver_apid, 16, msg);

		return res;
	}

	return FALSE;
}

/*
** AV_VIEW
*/
int av_view(char *file)
{
	int msg[8] = {AV_VIEW,0,0,0,0,0,0,0};

	msg[1] = gl_apid;
	if (file != NULL)
	{
		strncpy(av_buffer, file, 255);
		AVSTR2MSG(msg, 3, av_buffer);

		return appl_write(gl_avserver_apid, 16, msg);
	}

	return FALSE;
}


/*
** AV_OPENWIND
*/
int av_openwind(char *path, char *mask)
{
	int msg[8] = {AV_OPENWIND,0,0,0,0,0,0,0}, mask_offs, i;
	
	sprintf(av_buffer, path);

	/*
	** Kuttar av strengen ved siste \.
	*/
	i = (int) strlen(av_buffer);
	for (; av_buffer[i] != '\\'; av_buffer[i--] = '\0');	

	mask_offs = (int) strlen(av_buffer) + 1;
	sprintf(av_buffer + mask_offs, mask);
	
	msg[1] = gl_apid;
	AVSTR2MSG(msg, 3, av_buffer);
	AVSTR2MSG(msg, 5, av_buffer + mask_offs);
	return appl_write(gl_avserver_apid, (int) sizeof(msg), msg);
}

/*
** Melde seg som AV-klient
*/
static int av_protokoll(int supported)
{
	char server[9] = {'\0','\0','\0','\0','\0','\0','\0','\0','\0'};
	int msg[8] = {AV_PROTOKOLL,0,0,0,0,0,0,0};

	/*
	** Er ein AV-server innstallert?
	*/	
	gl_avserver_apid = -1;
	strcpy(server, getenv("AVSERVER"));
	if (server)
	{
		int i;
		
		for (i = 7; server[i] == '\0'; server[i--] = ' ');
		gl_avserver_apid = appl_find(server); 
		if (gl_avserver_apid < 0)
		{
			return -1;
		}
	}
	else
	{
		return -1;
	}

	/*
	** Jepp, gl_avserver_apid innstallert
	*/
	if (!av_buffer)
	{
		av_buffer = (char *)Mxalloc(8*1024, 0x0003|0x0028);
		if (!av_buffer)
		{
			return -1;
		}
	}

	strcpy(av_buffer, GEMDOSNAME);
	msg[1] = gl_apid;
	msg[3] = supported;
	AVSTR2MSG(msg, 6, av_buffer);

	return appl_write(gl_avserver_apid, 16, msg);
}

/*
** Melde seg for AV-serveren.
** 'supported' er ein bitmask som angir kva programmet stoettar.
** Sj thing.hyp for meir info.
*/
long av_find_server(short supported, int tries)
{
	int i, msg[8];

	/*
	** Lokalisere AVSERVER. "tries" forsoek m. eitt sekunds mellomrom.
	*/
	av_protokoll(supported);
	for (i = 1; i < tries; i++)
	{
		if (gl_avserver_apid == -1)
		{
			Fselect(1000, NULL, NULL, NULL);
			av_protokoll(supported);
		}
	}

	/*
	** Vente p svar fr AV-serveren
	*/
	if (gl_avserver_apid != -1)
	{
		msg[0] = 0;
		while (msg[0] != VA_PROTOSTATUS)
		{
			evnt_mesag(msg);
			if (msg[0] == AP_TERM)
			{
				if (av_buffer)
				{
					Mfree(av_buffer);
				}
				return -1;
			}
		}

		/*
		** Berre for  vere sikker...
		*/
		gl_avserver_apid = msg[1];

		/*
		** Returnerer 32-bit bitmask som angir kva av-serveren stoettar.
		** Sj thing.hyp for meir info.
		*/
		return (msg[3] & (msg[4] << 16));
	}

	if (av_buffer)
	{
		Mfree(av_buffer);
	}
	return -1;
}




/*
** Melde seg av
*/
int av_exit(void)
{
	int msg[8] = {AV_EXIT,0,0,0,0,0,0,0};
	
	if (av_buffer)
	{
		Mfree(av_buffer);
	}

	msg[1] = msg[3] = gl_apid;
	return appl_write(gl_avserver_apid, 16, msg);
}

/*
** Call ST-Guide
*/
int show_help(char *args)
{
	int msg[8] = {VA_START,0,0,0,0,0,0,0}, i, ok = FALSE;

	if (!av_buffer)
	{
		av_buffer = (char *)Mxalloc(8*1024, 0x0003|0x0028);
	}

	if (av_buffer != NULL)
	{
		if ( (i = appl_find("ST-GUIDE")) < 0)
		{
			if (getenv("STGUIDE") != NULL)
			{
				shel_write(0, 0, 0, getenv("STGUIDE"), "");
				if ( (i = appl_find("ST-GUIDE")) < 0)
				{
					ok = TRUE;
				}
			}
		}
		else
		{
			ok = TRUE;
		}
							
		if (ok)
		{
			strcpy(av_buffer, args);
			msg[1] = gl_apid;
			msg[3] = (int)(((long)av_buffer >> 16) & 0x0000ffff);
			msg[4] = (int)((long)av_buffer & 0x0000ffff);
			appl_write(i, 16, msg);

			return TRUE;
		}
	}
	
	return FALSE;
}
