/*
** This file is completely free, in every sense of the word.
** Do whatever you like with it, just beware that you use this
** code entirely at your own risk. I don't accept any liability
** whatsoever for any loss or damage caused by the use or
** misuse of this code.
**
** Jo Even Skarstein
** joska@nvg.org
** http://joska.nvg.org/
*/

#include "stdio.h"
#include "string.h"
#include "mintbind.h"
#include "conf.h"

#ifndef TRUE
#define FALSE 0
#define TRUE 1
#endif

#define MAX_ENTRIES 100
#define DEFAULT_VIEWER "c:\\accs\\1stguide.acc"

static VIEWER viewer[MAX_ENTRIES], *default_viewer = NULL;
static int entries = 0;

VIEWER *find_viewer(char *file);

/*
** Reads and processes the configuration file.
** Beware that there's no sanity checks in this code, and
** the format of the file is very strictly defined:
**
** Viewer[tab]Full path to viewer[tab]list of file-extensions[cr][lf]
** Example:
** 1ST-Guide	c:\1stguide.acc	jpg,jpe,jpeg
**
** No empty lines, comments etc. allowed. No unix- or mac-
** style linebreaks allowed. Always end file with a linebreak.
**
** Feel free to fix this. The current implementation is
** very rudimentary and not really suited for released
** software. Any fuck-ups in nugatti.inf can lead to endless
** loops and/or bus errors.
*/
int read_config(void)
{
	XATTR attr;
	static char *filebuffer, *t;
	long error;
	int handle;

	/*
	** Read the file
	*/
	if (Fxattr(0, CONFIG_FILE, &attr))
	{
		return FALSE;
	}

	filebuffer = t = Malloc(attr.size + 2);
	if (!filebuffer)
	{
		return FALSE;
	}
	
	error = Fopen(CONFIG_FILE, 0);
	if (error > 0)
	{
		handle = (int)error;
		error = Fread(handle, attr.size, filebuffer);
		filebuffer[attr.size] = '\0';
		Fclose(handle);
		if (error < 0)
		{
			return FALSE;
		}
	}
	else
	{
		return FALSE;
	}

	/*
	** Process it.
	*/
	while (*t != '\0' && entries < MAX_ENTRIES)
	{
		if (strncmpi(t, "NULL", 4))
			viewer[entries].viewer_name = t;
		else
			viewer[entries].viewer_name = NULL;
		while (*t++ != '\t'); t[-1] = '\0';
		viewer[entries].viewer = t;
		while (*t++ != '\t'); t[-1] = '\0';
		if (*t == '*') default_viewer = &viewer[entries];
		viewer[entries].filetypes = t;
		while (*t++ != 13); t[-1] = '\0';t++;

		#ifdef DEBUG
		if (viewer[entries].viewer_name) printf("\"%s\", ",  viewer[entries].viewer_name);
		printf("\"%s\", \"%s\"\n", viewer[entries].viewer, viewer[entries].filetypes);
		#endif

		strupr(viewer[entries].filetypes);
		entries++;
	}

	if (default_viewer)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

/*
** Returns a =< 3 long extension for any given file.
*/
char *fext(char *file)
{
	static char extension[4];
	VIEWER *v;
	
	v = find_viewer(file);
	if (v)
	{
		char *t = v->filetypes, *e = extension;

		for(; *t != ',' && *t != '\0' && t < v->filetypes + 4; *e++ = *t++); *e = '\0';
		#ifdef DEBUG
		printf("fext(): Found \"%s\"\n", extension);
		#endif
	}
	else
	{
		/*
		** If no match is found, just crop the extension to max
		** 3 chars
		*/
		long i;

		if (strstr(file, ".") == NULL)
		{
			return NULL;
		}
		for (i = strlen(file); file[i] != '.' && i > 0; i--);
		strncpy(extension, &file[i + 1], 3); extension[3] = '\0';
		strupr(extension);
		#ifdef DEBUG
		printf("fext(): Truncated: \"%s\"\n", extension);
		#endif
	}

	return extension;
}


/*
** Looks up the correct viewer and argumentstring for the filetype
** of file 'original_file'. 'view_file' is the file to pass on the
** argument string.
*/
void get_viewer(char *original_file, char *name, char *view_file, char *prog, char *cmd)
{
	VIEWER *v;
	
	v = find_viewer(original_file);
	if (!v) v = default_viewer;
	strcpy(prog, v->viewer);
	strcpy(cmd, view_file);
	if (v->viewer_name)
		strncpy(name, v->viewer_name, 64);
	else
		name[0] = '\0';

	#ifdef DEBUG
	printf("get_viewer(): Progname: \"%s\", Prog: \"%s\", Cmd: \"%s\"\n", name, prog, cmd);
	#endif
}

/*
** Search the configuration for the correct viewer for 'file'
*/
VIEWER *find_viewer(char *file)
{
	static char extension[65];
	size_t i;
	int t;

	/*
	** Find the extension
	*/
	#ifdef DEBUG
	printf("find_viewer(): Find extension\n");
	#endif
	if (strstr(file, ".") == NULL)
	{
		return NULL;
	}
	for (i = strlen(file); file[i] != '.' && i > 0; i--);
	strncpy(extension, &file[i + 1], 64); extension[64] = '\0';
	strupr(extension);
	#ifdef DEBUG
	printf("find_viewer(): Extension: %s\n", extension);
	#endif

	/*
	** Search the configuration-file for this extension.
	*/
	#ifdef DEBUG
	printf("find_viewer(): Search configuration\n");
	#endif
	for (t = 0; t < entries; t++)
	{
		char *found = viewer[t].filetypes - 1;

		#ifdef DEBUG
		if (found) printf("find_viewer(): t=%d, found=\"%s\"\n", t, found + 1);
		#endif
		while (found)
		{
			found++;
			found = strstr(found, extension);

			#ifdef DEBUG
			if (found) printf("find_viewer(): Searching: \"%s\"\n", found);
			#endif
			if (found && (found == viewer[t].filetypes || found[-1] == ',') && (found[strlen(extension)] == ',' || found[strlen(extension)] == '\0'))
			{
				return &viewer[t];
			}
		}
	}

	return NULL;
}