/*
** This file is completely free, in every sense of the word.
** Do whatever you like with it, just beware that you use this
** code entirely at your own risk. I don't accept any liability
** whatsoever for any loss or damage caused by the use or
** misuse of this code.
**
** Jo Even Skarstein
** joska@nvg.org
** http://joska.nvg.org/
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <aes.h>
#include <mintbind.h>
#include "quote.h"
#include "dhst.h"
#include "nugatti.h"
#include "av.h"
#include "conf.h"

int gl_apid;
extern int gl_avserver_apid;
long av_functions;
char tmp[1024];

#define AV_SUPPORTED 0x02 | 0x10 /* We understand VA_START and quoting */
#define NU_OPEN	0x1972
#define GET_VIEWER	0x1973

#define LINKDIR "U:\\RAM\\NUGATTI"

static int open_file(char *f);
static void split_arguments(char *arg, int *arg_c, char *arg_v[]);
static int trunc_filename(char *dest, char *source);

/*
** Initialise the application and wait for messages.
*/
int main(int argc, char *argv[])
{
	int msg[8] = {0,0,0,0,0,0,0,0}, msg_loop = TRUE, i, av_argc;
	short d;
	char *av_argv[100];
	static char nugatti[256];

	Pdomain(1);
	gl_apid = appl_init();
	if (gl_apid == -1)
	{
		printf("Nugatti: appl_init() failed.\n");
		return -1;
	}

	if (!read_config())
	{
		form_alert(1, "[1][Nugatti.inf is either missing|or corrupt.][OK, I'll fix it]");
		show_help("*:\\Nugatti.hyp 4 Configuration");
		Fselect(200,0,0,0);
		appl_exit();
		return 0;
	}

	menu_register(gl_apid, "  Nugatti");
	d = Dgetdrv();
	Dgetpath(tmp, d + 1);
	sprintf(nugatti, "%c:%s\\nugatti.app", (char)('A' + d), tmp);
	dhst_init("Nugatti", nugatti);

	Dcreate(LINKDIR);
	
	/*
	** Locate the AV-server and determine if it's up to the job.
	*/
	av_functions = av_find_server(AV_SUPPORTED, 5);
	if (av_functions == -1)
	{
		/*
		** AV-server not found.
		*/
		#ifdef DEBUG
		printf("AV server (%d) not found.\n", gl_avserver_apid);
		#endif
		appl_exit();
		return -1;
	}

	/*
	** Handle arguments if any.
	*/
	if (argc > 1)
	{
		int nu_msg[8] = {NU_OPEN};

		av_argc = argc - 1;
		for (i = 1; i < argc && i < 100; i++)
		{
			av_argv[i - 1] = argv[i];
			#ifdef DEBUG
			printf("argv[%d]: %s\n", i, argv[i]);
			#endif
		}

		appl_write(gl_apid, (int) sizeof(nu_msg), nu_msg);
	}
	
	/*
	** Enter the main msg-loop.
	*/
	while (msg_loop)
	{
		int av_lock = FALSE;
		static int i = 0;
		static char *t = NULL;

		evnt_mesag(msg);
		switch(msg[0])
		{
			case VA_START:
				#ifdef DEBUG
				printf("VA_START received: %s\n", AVMSG2STR(msg, 3));
				#endif
				if (!av_lock)
				{
					int nu_msg[8] = {NU_OPEN};

					t = malloc(strlen(AVMSG2STR(msg, 3)));
					strcpy(t, AVMSG2STR(msg, 3));
					split_arguments(t, &av_argc, av_argv);
					appl_write(gl_apid, (int) sizeof(nu_msg), nu_msg);
				}
				#ifdef DEBUG
				else printf("AV-buffer locked.\n");
				#endif

				break;

			case NU_OPEN:
				#ifdef DEBUG
				printf("NU_OPEN received: %s\n", av_argv[i]);
				#endif
				if (!av_lock)
				{
					av_lock = TRUE;
					open_file(av_argv[i++]);
				}
				break;

			case VA_PROGSTART:
				#ifdef DEBUG
				printf("VA_PROGSTART received\n");
				#endif
				av_lock = FALSE;
				if (i < av_argc && i < 100)
				{
					int nu_msg[8] = {NU_OPEN};
					appl_write(gl_apid, (int) sizeof(nu_msg), nu_msg);
				}
				else
				{
					av_argc = i = 0;
					free(t);
				}
				break;

			case DHST_ACK:
				#ifdef DEBUG
				printf("DHST_ACK received\n");
				#endif
				dhst_ack(AVMSG2STR(msg, 3));
				break;

			case GET_VIEWER:
				if (!av_lock)
				{
					VIEWER *v;
					
					v = find_viewer(AVMSG2STR(msg, 3));
					if (v)
					{
						extern char *av_buffer;
						strcpy(av_buffer, v->viewer);
					}
					else
					{
						av_buffer[0] = '\0';
					}
					AVSTR2MSG(msg, 3, av_buffer);
					msg[1] = gl_apid;
					appl_write(msg[1], (int) sizeof(msg), msg);
				}

			case AP_TERM:
				msg_loop = FALSE;
				break;
		}
	}	

	/*
	** Exit gracefully
	*/
	av_exit();
	appl_exit();

	return 0;
}

/*
** Split arguments. Beware that *arg is destroyed!
*/
static void split_arguments(char *arg, int *arg_c, char *arg_v[])
{
	int quoted = 0, i = 0;
	char *c = arg;

	while (*c != '\0' && i < 100)
	{
		/*
		** Skip any leading spaces
		*/
		for (;*c == ' '; c++);
		if (*c == '\0')
		{
			break;
		}

		/*
		** Add argument
		*/
		arg_v[i++] = c;
		
		while (*c != '\0')
		{
			/*
			** Check for quotes.
			*/
			if (*c == '\'')
			{
				if (c[1] == '\'')
				{
					/* Escaped ' */
					c++;
				}
				else
				{
					/* Toggle the quote-flag */
					quoted = 1 - quoted;
				}
			}

			/*
			** Split?
			*/
			if (!quoted && *c == ' ')
			{
				*c = '\0';
				c++;
				break;
			}

			c++;
		}
	}

	*arg_c = i;

	/*
	** Remove quote-marks
	*/
	for (i=0; i < *arg_c; i++)
	{
		char *t = tmp;

		unquote(t, arg_v[i]);
		strcpy(arg_v[i], t);
		#ifdef DEBUG
		printf("argv[%d]: \"%s\"\n", i, arg_v[i]);
		#endif
	}
}


/*
** Open file
*/
static int open_file(char *f)
{
	char *file = f;
	static char viewer[256], view_file[256], cmd[256], viewer_name[65], trunc[9];

	/*
	** Link the file to some place where the viewer can find
	** it. Give it a 3-character extension since it must fit
	** on a 8+3 partition.
	*/
	#ifdef DEBUG
	printf("open_file: fext()\n");
	#endif
	trunc_filename(trunc, file);
	if (fext(file))
	{
		sprintf(view_file, "%s\\%s.%s", LINKDIR, trunc, fext(file));
	}
	else
	{
		sprintf(view_file, "%s\\%s", LINKDIR, trunc);
	}
	Fdelete(view_file);
	Fsymlink(file, view_file);
	
	/*
	** Determine which viewer to use, and it's commandline.
	*/
	#ifdef DEBUG
	printf("open_file: get_viewer()\n");
	#endif
	get_viewer(file, viewer_name, view_file, viewer, cmd);
	
	/*
	** Start it via AV_STARTPROG
	*/
	#ifdef DEBUG
	printf("open_file: av_startprog()\n");
	#endif
	av_startprog(viewer, cmd);

	/*
	** Notify the DHST-server.
	*/
	if (viewer_name[0])
	{
		#ifdef DEBUG
		printf("open_file: dhst_add()\n");
		#endif
		if (dhst_add(file, file, viewer_name, NULL) == 0)
		{
			#ifdef DEBUG
			printf("DHST-buffer locked!\n");
			#endif
		}
	}

	return 0;
}


/*
** Truncate filenames to max. 8 TOS-legal characters.
** NB! This does not guarantee unique filenames!!
** "source" can be either just the filename with or without
** extension or a complete path with or without extension.
** "dest" must be atleast 9 chars long.
*/
static int trunc_filename(char *dest, char *source)
{
	char *s = source, *d = dest, *t;
	static char legal[] = {"ABCDEFGHIJKLMNOPQRSTUVWVXYZabcdefghijklmnopqrstuvwxyz0123456789!@#$%^&()+-=~`;'\",<>|[]_"};

	/*
	** Locate the first character of the actual filename
	*/
	t = s + strlen(s);
	while (t > s && *t != '\\') t--;
	if (*t == '\\') t++;
	s = t;
	
	/*
	** Copy max 8 legal characters to "dest"
	*/
	while(*s != '\0' && *s != '.' && d < dest + 8)
	{
		int i, l = (int) strlen(legal);

		for(i = 0; i < l; i++)
		{
			if (*s == legal[i])
			{
				*d++ = *s;
				break;
			}
		}
		s++;
	}
	*d = '\0';

	return TRUE;
}